//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x000000,//背景色
  },
}
//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Title: "assets/title-scene.png",
  Game: "assets/game-scene.png",
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  
  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new TitleScene());
  }
}

//タイトルシーン
class TitleScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Title].texture;
    this.addChild(img)

    const style = new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 40,
      fill: 0xffffff,
      fontWeight: 'bold',
      stroke: 0xFF0000,//縁取りの色
      strokeThickness: 4,//縁取りの太さ
    });
    const p = new PIXI.Text("おばけ", style);
    p.anchor.set(0.5, 0.5);
    p.position.set(Config.Screen.Width * 0.5, Config.Screen.Height * 0.15);
    this.addChild(p);

    this.isFading = true; 

    //モザイクフェードイン
    new MosaicIn(this, 4, 48, 4, 2, () => {
      this.isFading = false;
    });

    //タッチイベント
    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 
      
      //モザイクフェードアウト
      new MosaicOut(this, 4, 48, 4, 2, () => {
        core.replaceScene(new MainScene());
      });
      
    });
  }
  update(delta){
    super.update(delta);   
  }
}


//メインシーン
class MainScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵(取りあえず)
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Game].texture;
    this.addChild(img)

    this.isFading = true; 

    //モザイクフェードイン
    new MosaicIn(this, 4, 48, 4, 2, () => {
      this.isFading = false;
    });

    //タッチイベント
    this.on("pointerup", () => {
      if(this.isFading) return;
      this.isFading = true; 
  
      //モザイクフェードアウト
      new MosaicOut(this, 4, 48, 4, 2, () => {
        core.replaceScene(new TitleScene());
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}

//モザイクフェードアウト
class MosaicOut extends Container {
  constructor(container, min, max, interval, frequency, callback){
    super();
    container.addChild(this);
    this.size = min;
    this.max = max;
    this.interval = interval;//モザイクの大きさの間隔
    this.frequency = frequency;//モザイクアニメーションの間隔(フレームの間隔)
    this.count = 0;
    
    //コンテナからテクスチャー生成(画面を画像に)
    const texture = new PIXI.RenderTexture.create({width: Config.Screen.Width, height: Config.Screen.Height});
    core.app.renderer.render(container, texture);

    //モザイクに必要な最大量のスプライトを作成
    this.pixels = [];//スプライト保存用の配列
    this.iterate((x, y) => {
      const s = new EnchantSprite(1, 1);//1ドットだけのスプライト
      s.image = texture;
      s.frameNumber = y * Config.Screen.Width + x;
      s.scale.set(min, min);//拡大
      s.position.set(x, y);//表示位置
      this.addChild(s);
      this.pixels.push(s);//保存用配列に追加
    });

    //処理が終わったときに行う処理
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  //繰り返し文(同じ繰り返し処理があったのでメソッド化してみた)
  iterate(callback){
    for(let y = 0; y < Config.Screen.Height; y++){
      if(y % this.size != 0) continue;//指定サイズ毎の左上の座標だけ欲しいので
      for(let x = 0; x < Config.Screen.Width; x++){
        if(x % this.size != 0) continue;//指定サイズ毎の左上の座標だけ欲しいので
        callback(x, y);
      }
    }
  }
  update(delta){
    super.update(delta);
    if(this.count++ < this.frequency) return;//指定の時間間隔までは抜ける
    this.count = 0;
    this.size += this.interval;//モザイクのサイズを上げる
    if(this.size > this.max){//指定のサイズより大きくなったら終了
      this.onFinished();
      this.destroy();
      return;
    } 

    //モザイクのサイズと位置の変更
    let i = 0;
    this.iterate((x, y) => {
      const s = this.pixels[i++];
      s.frameNumber = y * Config.Screen.Width + x;
      s.scale.set(this.size, this.size);
      s.position.set(x, y);
    });

    //不要になったものは非表示(邪魔になるので)
    for(; i < this.pixels.length; i++){
      this.pixels[i].visible = false;
    }
  }
}

//モザイクフェードイン
class MosaicIn extends Container {
  constructor(container, min, max, interval, frequency, callback){
    super();
    container.addChild(this);
    this.size = min;
    this.min = min;
    this.max = max;
    this.interval = interval;
    this.frequency = frequency;
    this.count = 0;
    
    //コンテナからテクスチャー生成
    const texture = new PIXI.RenderTexture.create({width: Config.Screen.Width, height: Config.Screen.Height});
    core.app.renderer.render(container, texture);

    //必要最大量のスプライトを作成
    this.pixels = [];
    this.iterate((x, y) => {
      const s = new EnchantSprite(1, 1);
      s.image = texture;
      s.frameNumber = y * Config.Screen.Width + x;
      s.scale.set(min, min);
      s.position.set(x, y);
      this.addChild(s);
      this.pixels.push(s);
    });

    this.size = max;

    //最大値でモザイク表示
    let i = 0;
    this.iterate((x, y) => {
      const s = this.pixels[i++];
      s.frameNumber = y * Config.Screen.Width + x;
      s.scale.set(this.size, this.size);
      s.position.set(x, y);
    });
    //不要のものは一旦非表示
    for(; i < this.pixels.length; i++){
      this.pixels[i].visible = false;
    }

    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  //繰り返し文(同じ繰り返し処理があったのでメソッド化してみた)
  iterate(callback){
    for(let y = 0; y < Config.Screen.Height; y++){
      if(y % this.size != 0) continue;
      for(let x = 0; x < Config.Screen.Width; x++){
        if(x % this.size != 0) continue;
        callback(x, y);
      }
    }
  }
  update(delta){
    super.update(delta);
    if(this.count++ < this.frequency) return;
    this.count = 0;
    this.size -= this.interval;
    if(this.size < this.min){
      this.onFinished();
      this.destroy();
      return;
    }   
    let i = 0;
    this.iterate((x, y) => {
      const s = this.pixels[i++];
      s.frameNumber = y * Config.Screen.Width + x;
      s.scale.set(this.size, this.size);
      s.position.set(x, y);
      s.visible = true;
    });
  }
}